(***********************************************************************)
(*                                                                     *)
(*                              OCamlGen                               *)
(*                                                                     *)
(*                       OCaml module Generator                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 1997-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: og_types.mli,v 1.11 2014/01/31 09:51:52 fclement Exp $ *)

(*
(* Abstract syntax trees of data type definitions. *)

(* Lexems *) *)
type lexem =

   (* Operators *)
   | AMPERAMPER
   | QUESTION
   | QUOTE
   | LPAREN
   | LBRACKET
   | LBRACKETBAR
   | LBRACE
   | RPAREN
   | RBRACKET
   | RBRACE
   | BARRBRACKET
   | STAR
   | COLON
   | COMMA
   | DOT
   | DOTDOT
   | EXCEPTION
   | EXTERNAL
   | SEMI
   | SEMISEMI
   | LESS
   | LESSGREATER
   | LESSEQUAL
   | MINUS
   | MINUSGREATER
   | EQUAL
   | GREATER
   | GREATEREQUAL
   | BANGEQUAL
   | EQUALEQUAL
   | BAR
   | BARBAR
   | TILDA_OP of string
   | BANG_OP of string
   | AT_OP of string
   | PERCENT_OP of string
   | HAT_OP of string
   | AMPER_OP of string
   | STAR_OP of string
   | MINUS_OP of string
   | EQUAL_OP of string
   | PLUS_OP of string
   | LESS_OP of string
   | GREATER_OP of string
   | SLASH_OP of string
   | QUESTION_OP of string

   | TRUE | FALSE

   | MOD
   | OF | HTML | MUTABLE | TYPE | AND | BASE | PRIVATE

   | OPEN | INCLUDE | VAL
   | MODULE | SIG | END

   | INT of int | FLOAT of string | STRING of string | CHAR of char
   | COMMENT of string
   | LIDENT of string
   | UIDENT of string

   | EOF
;;

(*(* Abstract syntax trees of data type definitions. *)*)
type file = phrase list

and phrase =
  | Comment of string
  | Directive of dir
  | Type_definition of type_defs

and dir =
  | Dir_open of string
  | Dir_include of string

and type_defs = type_def list

and type_def = (type_name * type_arguments * type_kind)

and type_name = string

and type_arguments = string list

and type_kind =
  | Record_type of visibility * record_fields
  | Sum_type of visibility * sum_fields
  | Alias_type of visibility * alias_field
  | External_type

and visibility =
  | Public
  | Private
  | Abstract

and record_fields = record_field list

and record_field =
  | Field of record_label * type_expr
  | Field_mutable of record_label * type_expr

and record_label = string

and sum_fields = sum_field list

and sum_field =
  | Sum_tag of sum_label
  | Sum_inj of sum_label * type_expr

and sum_label = string

and alias_field = type_expr * type_kind option

and type_expr =
  | Var_type of string
  | Const_type of type_longident
  | Apply_type of type_longident * type_expr array
  | Tuple_type of type_expr array
  | Function_type of type_expr array
  | Paren_type of type_expr

and longident =
  | Lident of string
  | Ldot of longident * string
  | Lapply of longident * longident

and type_longident = longident
;;

(*(*
and html_annotation = Html_annotation of string

and ranges =
  | Basic_range of basic_range
  | Lt_range


and basic_range =
  | Int_range of int * int
  | Float_range of float * float
  | String_range of string * string
  | Char_range of char * char
*)*)
